@@ -8,18 +8,18 @@
 // <int>.<int>.<int>-<any text>.
 // or else upgrading old project file will not work
 // reliably.
-#define VERSION "3.6.0-Noosa"
+#define VERSION "3.8.3-Zanzibar"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 360
-#define VERSION_INT 30600
-#define ABISYM(x) x ## 30600
+//#define VERSION_INT 383
+#define VERSION_INT 30803
+#define ABISYM(x) x ## 30803
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Noosa"
+#define RELEASE_NAME "Zanzibar"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -27,11 +27,11 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr/local"
-#define CMAKE_SOURCE_DIR "/usr/obj/ports/qgis-3.6.0/qgis-3.6.0"
+#define CMAKE_SOURCE_DIR "/usr/obj/ports/qgis-3.8.3/qgis-3.8.3"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
-#define QSCINTILLA_VERSION_STR "2.10.1"
+#define QSCINTILLA_VERSION_STR "2.11.2"
 
 #if defined( __APPLE__ )
 //used by Mac to find system or bundle resources relative to amount of bundling
