@@ -27,7 +27,13 @@ from qgis.PyQt.QtWidgets import QApplication, QAction, QMenu, QInputDialog, QMes
 from qgis.PyQt.QtGui import QKeySequence, QIcon
 
 from qgis.gui import QgsMessageBar
-from qgis.core import Qgis, QgsApplication, QgsSettings
+from qgis.core import (
+    Qgis,
+    QgsApplication,
+    QgsSettings,
+    QgsMapLayerType,
+    QgsWkbTypes
+)
 from ..db_plugins import createDbPlugin
 
 
@@ -50,9 +56,6 @@ class BaseError(Exception):
     def __unicode__(self):
         return self.msg
 
-    def __str__(self):
-        return str(self).encode('utf-8')
-
 
 class InvalidDataException(BaseError):
     pass
@@ -276,7 +279,7 @@ class Database(DbItemObject):
         return "row_number() over ()"
 
     def toSqlLayer(self, sql, geomCol, uniqueCol, layerName="QueryLayer", layerType=None, avoidSelectById=False, filter=""):
-        from qgis.core import QgsMapLayer, QgsVectorLayer, QgsRasterLayer
+        from qgis.core import QgsVectorLayer, QgsRasterLayer
 
         if uniqueCol is None:
             if hasattr(self, 'uniqueIdFunction'):
@@ -293,7 +296,7 @@ class Database(DbItemObject):
         if avoidSelectById:
             uri.disableSelectAtId(True)
         provider = self.dbplugin().providerName()
-        if layerType == QgsMapLayer.RasterLayer:
+        if layerType == QgsMapLayerType.RasterLayer:
             return QgsRasterLayer(uri.uri(False), layerName, provider)
         return QgsVectorLayer(uri.uri(False), layerName, provider)
 
@@ -597,6 +600,7 @@ class Schema(DbItemObject):
         ret = self.database().connector.renameSchema(self.name, new_name)
         if ret is not False:
             self.name = new_name
+            # FIXME: refresh triggers
             self.refresh()
         return ret
 
@@ -655,6 +659,9 @@ class Table(DbItemObject):
         ret = self.database().connector.renameTable((self.schemaName(), self.name), new_name)
         if ret is not False:
             self.name = new_name
+            self._triggers = None
+            self._rules = None
+            self._constraints = None
             self.refresh()
         return ret
 
@@ -1098,13 +1105,17 @@ class TableField(TableSubItemObject):
             txt += u" DEFAULT %s" % self.default2String()
         return txt
 
+    def getComment(self):
+        """Returns the comment for a field"""
+        return ''
+
     def delete(self):
         return self.table().deleteField(self)
 
     def rename(self, new_name):
         return self.update(new_name)
 
-    def update(self, new_name, new_type_str=None, new_not_null=None, new_default_str=None):
+    def update(self, new_name, new_type_str=None, new_not_null=None, new_default_str=None, new_comment=None):
         self.table().aboutToChange.emit()
         if self.name == new_name:
             new_name = None
@@ -1114,10 +1125,12 @@ class TableField(TableSubItemObject):
             new_not_null = None
         if self.default2String() == new_default_str:
             new_default_str = None
-
+        if self.comment == new_comment:
+            # Update also a new_comment
+            new_comment = None
         ret = self.table().database().connector.updateTableColumn((self.table().schemaName(), self.table().name),
-                                                                  self.name, new_name, new_type_str, new_not_null,
-                                                                  new_default_str)
+                                                                  self.name, new_name, new_type_str,
+                                                                  new_not_null, new_default_str, new_comment)
         if ret is not False:
             self.table().refreshFields()
         return ret
