@@ -25,12 +25,11 @@ from builtins import str
 from builtins import range
 
 from qgis.PyQt.QtCore import Qt, QFileInfo
-from qgis.PyQt.QtWidgets import QDialog, QFileDialog, QMessageBox, QApplication
-from qgis.PyQt.QtGui import QCursor
+from qgis.PyQt.QtWidgets import QDialog, QFileDialog, QMessageBox
 
 from qgis.core import (QgsDataSourceUri,
                        QgsVectorLayer,
-                       QgsMapLayer,
+                       QgsMapLayerType,
                        QgsProviderRegistry,
                        QgsCoordinateReferenceSystem,
                        QgsVectorLayerExporter,
@@ -52,6 +51,11 @@ class DlgImportVector(QDialog, Ui_Dialog):
         self.outUri = outUri
         self.setupUi(self)
 
+        supportCom = self.db.supportsComment()
+        if not supportCom:
+            self.chkCom.setVisible(False)
+            self.editCom.setVisible(False)
+
         self.default_pk = "id"
         self.default_geom = "geom"
 
@@ -70,6 +74,7 @@ class DlgImportVector(QDialog, Ui_Dialog):
         self.cboSchema.currentIndexChanged.connect(self.populateTables)
         self.widgetSourceSrid.setCrs(QgsProject.instance().crs())
         self.widgetTargetSrid.setCrs(QgsProject.instance().crs())
+        self.updateInputLayer()
 
     def setupWorkingMode(self, mode):
         """ hide the widget to select a layer/file if the input layer is already set """
@@ -80,9 +85,7 @@ class DlgImportVector(QDialog, Ui_Dialog):
 
         if mode == self.ASK_FOR_INPUT_MODE:
             self.btnChooseInputFile.clicked.connect(self.chooseInputFile)
-            # self.cboInputLayer.lineEdit().editingFinished.connect(self.updateInputLayer)
-            self.cboInputLayer.editTextChanged.connect(self.inputPathChanged)
-            # self.cboInputLayer.currentIndexChanged.connect(self.updateInputLayer)
+            self.cboInputLayer.currentTextChanged.connect(self.updateInputLayer)
             self.btnUpdateInputLayer.clicked.connect(self.updateInputLayer)
 
             self.editPrimaryKey.setText(self.default_pk)
@@ -130,7 +133,7 @@ class DlgImportVector(QDialog, Ui_Dialog):
         for nodeLayer in QgsProject.instance().layerTreeRoot().findLayers():
             layer = nodeLayer.layer()
             # TODO: add import raster support!
-            if layer.type() == QgsMapLayer.VectorLayer:
+            if layer.type() == QgsMapLayerType.VectorLayer:
                 self.cboInputLayer.addItem(layer.name(), layer.id())
 
     def deleteInputLayer(self):
@@ -158,15 +161,8 @@ class DlgImportVector(QDialog, Ui_Dialog):
         settings.setValue("/db_manager/lastUsedDir", QFileInfo(filename).filePath())
         settings.setValue("/UI/lastVectorFileFilter", lastVectorFormat)
 
-        self.cboInputLayer.setEditText(filename)
-
-    def inputPathChanged(self, path):
-        if self.cboInputLayer.currentIndex() < 0:
-            return
-        self.cboInputLayer.blockSignals(True)
         self.cboInputLayer.setCurrentIndex(-1)
-        self.cboInputLayer.setEditText(path)
-        self.cboInputLayer.blockSignals(False)
+        self.cboInputLayer.setEditText(filename)
 
     def reloadInputLayer(self):
         """ create the input layer and update available options """
@@ -183,7 +179,7 @@ class DlgImportVector(QDialog, Ui_Dialog):
 
             layerName = QFileInfo(filename).completeBaseName()
             layer = QgsVectorLayer(filename, layerName, "ogr")
-            if not layer.isValid() or layer.type() != QgsMapLayer.VectorLayer:
+            if not layer.isValid() or layer.type() != QgsMapLayerType.VectorLayer:
                 layer.deleteLater()
                 return False
 
@@ -378,6 +374,13 @@ class DlgImportVector(QDialog, Ui_Dialog):
         if self.chkSpatialIndex.isEnabled() and self.chkSpatialIndex.isChecked():
             self.db.connector.createSpatialIndex((schema, table), geom)
 
+        # add comment on table
+        supportCom = self.db.supportsComment()
+        if self.chkCom.isEnabled() and self.chkCom.isChecked() and supportCom:
+            # using connector executing COMMENT ON TABLE query (with editCome.text() value)
+            com = self.editCom.text()
+            self.db.connector.commentTable(schema, table, com)
+
         self.db.connection().reconnect()
         self.db.refresh()
         QMessageBox.information(self, self.tr("Import to Database"), self.tr("Import was successful."))
