@@ -21,13 +21,26 @@ The content of this file is based on
  *                                                                         *
  ***************************************************************************/
 """
+from builtins import zip
+from builtins import next
+from builtins import str
+from hashlib import md5
 
-from PyQt4.QtCore import Qt, QObject, QSettings, QByteArray, SIGNAL, pyqtSignal
-from PyQt4.QtGui import QDialog, QWidget, QAction, QKeySequence, \
-    QDialogButtonBox, QApplication, QCursor, QMessageBox, QClipboard, QInputDialog, QIcon, QStyledItemDelegate, QStandardItemModel, QStandardItem
-from PyQt4.Qsci import QsciAPIs
+import os
 
-from qgis.core import QgsProject
+from qgis.PyQt.QtCore import Qt, pyqtSignal, QDir
+from qgis.PyQt.QtWidgets import QDialog, QWidget, QAction, QApplication, QInputDialog, QStyledItemDelegate, QTableWidgetItem, QFileDialog
+from qgis.PyQt.QtGui import QKeySequence, QCursor, QClipboard, QIcon, QStandardItemModel, QStandardItem
+from qgis.PyQt.Qsci import QsciAPIs
+
+from qgis.core import (
+    QgsProject,
+    QgsApplication,
+    QgsTask,
+    QgsSettings,
+    QgsMapLayerType
+)
+from qgis.utils import OverrideCursor
 
 from .db_plugins.plugin import BaseError
 from .db_plugins.postgis.plugin import PGDatabase
@@ -35,7 +48,7 @@ from .dlg_db_error import DlgDbError
 from .dlg_query_builder import QueryBuilderDlg
 
 try:
-    from qgis.gui import QgsCodeEditorSQL
+    from qgis.gui import QgsCodeEditorSQL  # NOQA
 except:
     from .sqledit import SqlEdit
     from qgis import gui
@@ -49,29 +62,55 @@ import re
 
 class DlgSqlWindow(QWidget, Ui_Dialog):
     nameChanged = pyqtSignal(str)
+    QUERY_HISTORY_LIMIT = 20
 
     def __init__(self, iface, db, parent=None):
         QWidget.__init__(self, parent)
+        self.mainWindow = parent
         self.iface = iface
         self.db = db
+        self.dbType = db.connection().typeNameString()
+        self.connectionName = db.connection().connectionName()
         self.filter = ""
-        self.allowMultiColumnPk = isinstance(db, PGDatabase) # at the moment only PostgreSQL allows a primary key to span multiple columns, spatialite doesn't
-        self.aliasSubQuery = isinstance(db, PGDatabase)	# only PostgreSQL requires subqueries to be aliases
+        self.modelAsync = None
+        self.allowMultiColumnPk = isinstance(db, PGDatabase)  # at the moment only PostgreSQL allows a primary key to span multiple columns, SpatiaLite doesn't
+        self.aliasSubQuery = isinstance(db, PGDatabase)       # only PostgreSQL requires subqueries to be aliases
         self.setupUi(self)
         self.setWindowTitle(
-            u"%s - %s [%s]" % (self.windowTitle(), db.connection().connectionName(), db.connection().typeNameString()))
+            self.tr(u"{0} - {1} [{2}]").format(self.windowTitle(), self.connectionName, self.dbType))
 
-        self.defaultLayerName = 'QueryLayer'
+        self.defaultLayerName = self.tr('QueryLayer')
 
         if self.allowMultiColumnPk:
-            self.uniqueColumnCheck.setText(self.trUtf8("Column(s) with unique values"))
+            self.uniqueColumnCheck.setText(self.tr("Column(s) with unique values"))
         else:
-            self.uniqueColumnCheck.setText(self.trUtf8("Column with unique values"))
+            self.uniqueColumnCheck.setText(self.tr("Column with unique values"))
 
         self.editSql.setFocus()
         self.editSql.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
+        self.editSql.setMarginVisible(True)
         self.initCompleter()
 
+        settings = QgsSettings()
+        self.history = settings.value('DB_Manager/queryHistory/' + self.dbType, {self.connectionName: []})
+        if self.connectionName not in self.history:
+            self.history[self.connectionName] = []
+
+        self.queryHistoryWidget.setVisible(False)
+        self.queryHistoryTableWidget.verticalHeader().hide()
+        self.queryHistoryTableWidget.doubleClicked.connect(self.insertQueryInEditor)
+        self.populateQueryHistory()
+        self.btnQueryHistory.toggled.connect(self.showHideQueryHistory)
+
+        self.btnCancel.setEnabled(False)
+        self.btnCancel.clicked.connect(self.executeSqlCanceled)
+        self.btnCancel.setShortcut(QKeySequence.Cancel)
+        self.progressBar.setEnabled(False)
+        self.progressBar.setRange(0, 100)
+        self.progressBar.setValue(0)
+        self.progressBar.setFormat("")
+        self.progressBar.setAlignment(Qt.AlignCenter)
+
         # allow copying results
         copyAction = QAction("copy", self)
         self.viewResult.addAction(copyAction)
@@ -84,6 +123,8 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         self.btnClear.clicked.connect(self.clearSql)
 
         self.presetStore.clicked.connect(self.storePreset)
+        self.presetSaveAsFile.clicked.connect(self.saveAsFilePreset)
+        self.presetLoadFile.clicked.connect(self.loadFilePreset)
         self.presetDelete.clicked.connect(self.deletePreset)
         self.presetCombo.activated[str].connect(self.loadPreset)
         self.presetCombo.activated[str].connect(self.presetName.setText)
@@ -99,8 +140,8 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         self.uniqueCombo.setModel(self.uniqueModel)
         if self.allowMultiColumnPk:
             self.uniqueCombo.setItemDelegate(QStyledItemDelegate())
-            self.uniqueModel.itemChanged.connect(self.uniqueChanged)                # react to the (un)checking of an item
-            self.uniqueCombo.lineEdit().textChanged.connect(self.uniqueTextChanged) # there are other events that change the displayed text and some of them can not be caught directly
+            self.uniqueModel.itemChanged.connect(self.uniqueChanged)                 # react to the (un)checking of an item
+            self.uniqueCombo.lineEdit().textChanged.connect(self.uniqueTextChanged)  # there are other events that change the displayed text and some of them can not be caught directly
 
         # hide the load query as layer if feature is not supported
         self._loadAsLayerAvailable = self.db.connector.hasCustomQuerySupport()
@@ -123,6 +164,48 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
 
         self.presetName.textChanged.connect(self.nameChanged)
 
+    def insertQueryInEditor(self, item):
+        sql = item.data(Qt.DisplayRole)
+        self.editSql.insertText(sql)
+
+    def showHideQueryHistory(self, visible):
+        self.queryHistoryWidget.setVisible(visible)
+
+    def populateQueryHistory(self):
+        self.queryHistoryTableWidget.clearContents()
+        self.queryHistoryTableWidget.setRowCount(0)
+        dictlist = self.history[self.connectionName]
+
+        if not dictlist:
+            return
+
+        for i in range(len(dictlist)):
+            self.queryHistoryTableWidget.insertRow(0)
+            queryItem = QTableWidgetItem(dictlist[i]['query'])
+            rowsItem = QTableWidgetItem(str(dictlist[i]['rows']))
+            durationItem = QTableWidgetItem(str(dictlist[i]['secs']))
+            self.queryHistoryTableWidget.setItem(0, 0, queryItem)
+            self.queryHistoryTableWidget.setItem(0, 1, rowsItem)
+            self.queryHistoryTableWidget.setItem(0, 2, durationItem)
+
+        self.queryHistoryTableWidget.resizeColumnsToContents()
+        self.queryHistoryTableWidget.resizeRowsToContents()
+
+    def writeQueryHistory(self, sql, affectedRows, secs):
+        if len(self.history[self.connectionName]) >= self.QUERY_HISTORY_LIMIT:
+            self.history[self.connectionName].pop(0)
+
+        settings = QgsSettings()
+        self.history[self.connectionName].append({'query': sql,
+                                                  'rows': affectedRows,
+                                                  'secs': secs})
+        settings.setValue('DB_Manager/queryHistory/' + self.dbType, self.history)
+
+        self.populateQueryHistory()
+
+    def getQueryHash(self, name):
+        return 'q%s' % md5(name.encode('utf8')).hexdigest()
+
     def updatePresetsCombobox(self):
         self.presetCombo.clear()
 
@@ -140,9 +223,9 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         query = self._getSqlQuery()
         if query == "":
             return
-        name = self.presetName.text()
-        QgsProject.instance().writeEntry('DBManager', 'savedQueries/q' + unicode(name.__hash__()) + '/name', name)
-        QgsProject.instance().writeEntry('DBManager', 'savedQueries/q' + unicode(name.__hash__()) + '/query', query)
+        name = str(self.presetName.text())
+        QgsProject.instance().writeEntry('DBManager', 'savedQueries/' + self.getQueryHash(name) + '/name', name)
+        QgsProject.instance().writeEntry('DBManager', 'savedQueries/' + self.getQueryHash(name) + '/query', query)
         index = self.presetCombo.findText(name)
         if index == -1:
             self.presetCombo.addItem(name)
@@ -150,15 +233,55 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         else:
             self.presetCombo.setCurrentIndex(index)
 
+    def saveAsFilePreset(self):
+        settings = QgsSettings()
+        lastDir = settings.value('DB_Manager/lastDirSQLFIle', "")
+
+        query = self.editSql.text()
+        if query == "":
+            return
+
+        filename, _ = QFileDialog.getSaveFileName(
+            self,
+            self.tr('Save SQL Query'),
+            lastDir,
+            self.tr("SQL File (*.sql, *.SQL)"))
+
+        if filename:
+            if not filename.lower().endswith('.sql'):
+                filename += ".sql"
+
+            with open(filename, 'w') as f:
+                f.write(query)
+                lastDir = os.path.dirname(filename)
+                settings.setValue('DB_Manager/lastDirSQLFile', lastDir)
+
+    def loadFilePreset(self):
+        settings = QgsSettings()
+        lastDir = settings.value('DB_Manager/lastDirSQLFIle', "")
+
+        filename, _ = QFileDialog.getOpenFileName(
+            self,
+            self.tr("Load SQL Query"),
+            lastDir,
+            self.tr("SQL File (*.sql, *.SQL)"))
+
+        if filename:
+            with open(filename, 'r') as f:
+                self.editSql.clear()
+                for line in f:
+                    self.editSql.insertText(line)
+                lastDir = os.path.dirname(filename)
+                settings.setValue('DB_Manager/lastDirSQLFile', lastDir)
+
     def deletePreset(self):
         name = self.presetCombo.currentText()
-        QgsProject.instance().removeEntry('DBManager', 'savedQueries/q' + unicode(name.__hash__()))
+        QgsProject.instance().removeEntry('DBManager', 'savedQueries/' + self.getQueryHash(name))
         self.presetCombo.removeItem(self.presetCombo.findText(name))
         self.presetCombo.setCurrentIndex(-1)
 
     def loadPreset(self, name):
-        query = QgsProject.instance().readEntry('DBManager', 'savedQueries/q' + unicode(name.__hash__()) + '/query')[0]
-        name = QgsProject.instance().readEntry('DBManager', 'savedQueries/q' + unicode(name.__hash__()) + '/name')[0]
+        query = QgsProject.instance().readEntry('DBManager', 'savedQueries/' + self.getQueryHash(name) + '/query')[0]
         self.editSql.setText(query)
 
     def loadAsLayerToggled(self, checked):
@@ -172,44 +295,89 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         self.editSql.setFocus()
         self.filter = ""
 
+    def updateUiWhileSqlExecution(self, status):
+        if status:
+            for i in range(0, self.mainWindow.tabs.count()):
+                if i != self.mainWindow.tabs.currentIndex():
+                    self.mainWindow.tabs.setTabEnabled(i, False)
+
+            self.mainWindow.menuBar.setEnabled(False)
+            self.mainWindow.toolBar.setEnabled(False)
+            self.mainWindow.tree.setEnabled(False)
+
+            for w in self.findChildren(QWidget):
+                w.setEnabled(False)
+
+            self.btnCancel.setEnabled(True)
+            self.progressBar.setEnabled(True)
+            self.progressBar.setRange(0, 0)
+        else:
+            for i in range(0, self.mainWindow.tabs.count()):
+                if i != self.mainWindow.tabs.currentIndex():
+                    self.mainWindow.tabs.setTabEnabled(i, True)
+
+            self.mainWindow.refreshTabs()
+            self.mainWindow.menuBar.setEnabled(True)
+            self.mainWindow.toolBar.setEnabled(True)
+            self.mainWindow.tree.setEnabled(True)
+
+            for w in self.findChildren(QWidget):
+                w.setEnabled(True)
+
+            self.btnCancel.setEnabled(False)
+            self.progressBar.setRange(0, 100)
+            self.progressBar.setEnabled(False)
+
+    def executeSqlCanceled(self):
+        self.btnCancel.setEnabled(False)
+        self.modelAsync.cancel()
+
+    def executeSqlCompleted(self):
+        self.updateUiWhileSqlExecution(False)
+
+        with OverrideCursor(Qt.WaitCursor):
+            if self.modelAsync.task.status() == QgsTask.Complete:
+                model = self.modelAsync.model
+                quotedCols = []
+
+                self.viewResult.setModel(model)
+                self.lblResult.setText(self.tr("{0} rows, {1:.3f} seconds").format(model.affectedRows(), model.secs()))
+                cols = self.viewResult.model().columnNames()
+                for col in cols:
+                    quotedCols.append(self.db.connector.quoteId(col))
+
+                self.setColumnCombos(cols, quotedCols)
+
+                self.writeQueryHistory(self.modelAsync.task.sql, model.affectedRows(), model.secs())
+                self.update()
+            elif not self.modelAsync.canceled:
+                DlgDbError.showError(self.modelAsync.error, self)
+                self.uniqueModel.clear()
+                self.geomCombo.clear()
+
     def executeSql(self):
 
-        sql = self._getSqlQuery()
+        sql = self._getExecutableSqlQuery()
         if sql == "":
             return
 
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-
         # delete the old model
         old_model = self.viewResult.model()
         self.viewResult.setModel(None)
         if old_model:
             old_model.deleteLater()
 
-        cols = []
-        quotedCols = []
-
         try:
-            # set the new model
-            model = self.db.sqlResultModel(sql, self)
-            self.viewResult.setModel(model)
-            self.lblResult.setText(self.tr("%d rows, %.1f seconds") % (model.affectedRows(), model.secs()))
-            cols = self.viewResult.model().columnNames()
-            for col in cols:
-                quotedCols.append(self.db.connector.quoteId(col))
-
-        except BaseError as e:
-            QApplication.restoreOverrideCursor()
+            self.modelAsync = self.db.sqlResultModelAsync(sql, self)
+            self.modelAsync.done.connect(self.executeSqlCompleted)
+            self.updateUiWhileSqlExecution(True)
+            QgsApplication.taskManager().addTask(self.modelAsync.task)
+        except Exception as e:
             DlgDbError.showError(e, self)
             self.uniqueModel.clear()
             self.geomCombo.clear()
             return
 
-        self.setColumnCombos(cols, quotedCols)
-
-        self.update()
-        QApplication.restoreOverrideCursor()
-
     def _getSqlLayer(self, _filter):
         hasUniqueField = self.uniqueColumnCheck.checkState() == Qt.Checked
         if hasUniqueField:
@@ -231,7 +399,7 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         else:
             geomFieldName = None
 
-        query = self._getSqlQuery()
+        query = self._getExecutableSqlQuery()
         if query == "":
             return None
 
@@ -239,13 +407,11 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         if query.strip().endswith(';'):
             query = query.strip()[:-1]
 
-        from qgis.core import QgsMapLayer, QgsMapLayerRegistry
-
-        layerType = QgsMapLayer.VectorLayer if self.vectorRadio.isChecked() else QgsMapLayer.RasterLayer
+        layerType = QgsMapLayerType.VectorLayer if self.vectorRadio.isChecked() else QgsMapLayerType.RasterLayer
 
         # get a new layer name
         names = []
-        for layer in QgsMapLayerRegistry.instance().mapLayers().values():
+        for layer in list(QgsProject.instance().mapLayers().values()):
             names.append(layer.name())
 
         layerName = self.layerNameEdit.text()
@@ -263,74 +429,67 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
         if layer.isValid():
             return layer
         else:
+            e = BaseError(self.tr("There was an error creating the SQL layer, please check the logs for further information."))
+            DlgDbError.showError(e, self)
             return None
 
     def loadSqlLayer(self):
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-        try:
+        with OverrideCursor(Qt.WaitCursor):
             layer = self._getSqlLayer(self.filter)
-            if layer == None:
+            if layer is None:
                 return
 
-            from qgis.core import QgsMapLayerRegistry
-            QgsMapLayerRegistry.instance().addMapLayers([layer], True)
-        finally:
-            QApplication.restoreOverrideCursor()
+            QgsProject.instance().addMapLayers([layer], True)
 
     def fillColumnCombos(self):
-        query = self._getSqlQuery()
+        query = self._getExecutableSqlQuery()
         if query == "":
             return
 
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
+        with OverrideCursor(Qt.WaitCursor):
+            # remove a trailing ';' from query if present
+            if query.strip().endswith(';'):
+                query = query.strip()[:-1]
 
-        # remove a trailing ';' from query if present
-        if query.strip().endswith(';'):
-            query = query.strip()[:-1]
+            # get all the columns
+            quotedCols = []
+            connector = self.db.connector
+            if self.aliasSubQuery:
+                # get a new alias
+                aliasIndex = 0
+                while True:
+                    alias = "_subQuery__%d" % aliasIndex
+                    escaped = re.compile('\\b("?)' + re.escape(alias) + '\\1\\b')
+                    if not escaped.search(query):
+                        break
+                    aliasIndex += 1
 
-        # get all the columns
-        cols = []
-        quotedCols = []
-        connector = self.db.connector
-        if self.aliasSubQuery:
-            # get a new alias
-            aliasIndex = 0
-            while True:
-                alias = "_subQuery__%d" % aliasIndex
-                escaped = re.compile('\\b("?)' + re.escape(alias) + '\\1\\b')
-                if not escaped.search(query):
-                    break
-                aliasIndex += 1
+                sql = u"SELECT * FROM (%s\n) AS %s LIMIT 0" % (str(query), connector.quoteId(alias))
+            else:
+                sql = u"SELECT * FROM (%s\n) WHERE 1=0" % str(query)
 
-            sql = u"SELECT * FROM (%s\n) AS %s LIMIT 0" % (unicode(query), connector.quoteId(alias))
-        else:
-            sql = u"SELECT * FROM (%s\n) WHERE 1=0" % unicode(query)
+            c = None
+            try:
+                c = connector._execute(None, sql)
+                cols = connector._get_cursor_columns(c)
+                for col in cols:
+                    quotedCols.append(connector.quoteId(col))
 
-        c = None
-        try:
-            c = connector._execute(None, sql)
-            cols = connector._get_cursor_columns(c)
-            for col in cols:
-                quotedCols.append(connector.quoteId(col))
+            except BaseError as e:
+                DlgDbError.showError(e, self)
+                self.uniqueModel.clear()
+                self.geomCombo.clear()
+                return
 
-        except BaseError as e:
-            QApplication.restoreOverrideCursor()
-            DlgDbError.showError(e, self)
-            self.uniqueModel.clear()
-            self.geomCombo.clear()
-            return
+            finally:
+                if c:
+                    c.close()
+                    del c
 
-        finally:
-            if c:
-                c.close()
-                del c
-
-        self.setColumnCombos(cols, quotedCols)
-
-        QApplication.restoreOverrideCursor()
+            self.setColumnCombos(cols, quotedCols)
 
     def setColumnCombos(self, cols, quotedCols):
-        # get sensible default columns. do this before sorting in case there's hints in the column order (eg, id is more likely to be first)
+        # get sensible default columns. do this before sorting in case there's hints in the column order (e.g., id is more likely to be first)
         try:
             defaultGeomCol = next(col for col in cols if col in ['geom', 'geometry', 'the_geom', 'way'])
         except:
@@ -386,10 +545,6 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
                 items[0].setCheckState(Qt.Checked)
             else:
                 self.uniqueCombo.setEditText(defaultUniqueCol)
-        try:
-            pass
-        except:
-            pass
 
     def copySelectedResults(self):
         if len(self.viewResult.selectedIndexes()) <= 0:
@@ -415,7 +570,7 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
             dictionary = getSqlDictionary()
 
         wordlist = []
-        for name, value in dictionary.iteritems():
+        for value in dictionary.values():
             wordlist += value  # concat lists
         wordlist = list(set(wordlist))  # remove duplicates
 
@@ -434,10 +589,10 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
             self.editSql.setText(dlg.query)
 
     def createView(self):
-        name, ok = QInputDialog.getText(None, "View name", "View name")
+        name, ok = QInputDialog.getText(None, self.tr("View Name"), self.tr("View name"))
         if ok:
             try:
-                self.db.connector.createSpatialView(name, self._getSqlQuery())
+                self.db.connector.createSpatialView(name, self._getExecutableSqlQuery())
             except BaseError as e:
                 DlgDbError.showError(e, self)
 
@@ -447,6 +602,17 @@ class DlgSqlWindow(QWidget, Ui_Dialog):
             sql = self.editSql.text()
         return sql
 
+    def _getExecutableSqlQuery(self):
+        sql = self._getSqlQuery()
+
+        # Clean it up!
+        lines = []
+        for line in sql.split('\n'):
+            if not line.strip().startswith('--'):
+                lines.append(line)
+        sql = ' '.join(lines)
+        return sql.strip()
+
     def uniqueChanged(self):
         # when an item is (un)checked, simply trigger an update of the combobox text
         self.uniqueTextChanged(None)
