@@ -21,15 +21,12 @@ __author__ = 'Victor Olaya'
 __date__ = 'February 2015'
 __copyright__ = '(C) 2014-2015, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '58734527abdbaab95719a48a4a64ea05abd70b44'
-
 import stat
 import shutil
 import shlex
 import subprocess
 import os
+import sys
 
 from qgis.core import (Qgis,
                        QgsApplication,
@@ -221,7 +218,7 @@ class Grass7Utils:
                 if "GISBASE" in os.environ:
                     folder = os.environ["GISBASE"]
                 else:
-                    testfolder = os.path.join(str(QgsApplication.prefixPath()), 'grass')
+                    testfolder = os.path.join(os.path.dirname(QgsApplication.prefixPath()), 'grass')
                     if os.path.isdir(testfolder):
                         grassfolders = sorted([f for f in os.listdir(testfolder) if f.startswith("grass-7.") and os.path.isdir(os.path.join(testfolder, f))], reverse=True, key=lambda x: [int(v) for v in x[len("grass-"):].split('.') if v != 'svn'])
                         if grassfolders:
@@ -370,10 +367,14 @@ class Grass7Utils:
         # QgsMessageLog.logMessage('exec: {}'.format(command), 'DEBUG', Qgis.Info)
 
         # For MS-Windows, we need to hide the console window.
+        kw = {}
         if isWindows():
             si = subprocess.STARTUPINFO()
             si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
             si.wShowWindow = subprocess.SW_HIDE
+            kw['startupinfo'] = si
+            if sys.version_info >= (3, 6):
+                kw['encoding'] = "cp{}".format(Grass7Utils.getWindowsCodePage())
 
         with subprocess.Popen(
                 command,
@@ -383,8 +384,7 @@ class Grass7Utils:
                 stderr=subprocess.STDOUT,
                 universal_newlines=True,
                 env=grassenv,
-                encoding="cp{}".format(Grass7Utils.getWindowsCodePage()) if isWindows() else None,
-                startupinfo=si if isWindows() else None
+                **kw
         ) as proc:
             for line in iter(proc.stdout.readline, ''):
                 if 'GRASS_INFO_PERCENT' in line:
@@ -415,10 +415,14 @@ class Grass7Utils:
         if not grassOutDone and outputCommands:
             command, grassenv = Grass7Utils.prepareGrassExecution(outputCommands)
             # For MS-Windows, we need to hide the console window.
+            kw = {}
             if isWindows():
                 si = subprocess.STARTUPINFO()
                 si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                 si.wShowWindow = subprocess.SW_HIDE
+                kw['startupinfo'] = si
+                if sys.version_info >= (3, 6):
+                    kw['encoding'] = "cp{}".format(Grass7Utils.getWindowsCodePage())
             with subprocess.Popen(
                     command,
                     shell=False,
@@ -427,8 +431,7 @@ class Grass7Utils:
                     stderr=subprocess.STDOUT,
                     universal_newlines=True,
                     env=grassenv,
-                    encoding="cp{}".format(Grass7Utils.getWindowsCodePage()) if isWindows() else None,
-                    startupinfo=si if isWindows() else None
+                    **kw
             ) as proc:
                 for line in iter(proc.stdout.readline, ''):
                     if 'GRASS_INFO_PERCENT' in line:
