@@ -22,16 +22,12 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '9b176802e5512bc310765d8ebe4e44b5f4663453'
-
 import os
 import math
 
 from qgis.PyQt.QtCore import Qt, QPointF, QRectF
-from qgis.PyQt.QtGui import QFont, QFontMetricsF, QPen, QBrush, QColor, QPolygonF, QPicture, QPainter
-from qgis.PyQt.QtWidgets import QGraphicsItem, QMessageBox, QMenu
+from qgis.PyQt.QtGui import QFont, QFontMetricsF, QPen, QBrush, QColor, QPolygonF, QPicture, QPainter, QPalette
+from qgis.PyQt.QtWidgets import QApplication, QGraphicsItem, QMessageBox, QMenu
 from qgis.PyQt.QtSvg import QSvgRenderer
 from qgis.core import (QgsProcessingParameterDefinition,
                        QgsProcessingModelParameter,
@@ -51,10 +47,12 @@ class ModelerGraphicItem(QGraphicsItem):
 
     def __init__(self, element, model, controls, scene=None):
         super(ModelerGraphicItem, self).__init__(None)
+        self.setAcceptHoverEvents(True)
         self.controls = controls
         self.model = model
         self.scene = scene
         self.element = element
+        self.hover_over_item = False
         if isinstance(element, QgsProcessingModelParameter):
             svg = QSvgRenderer(os.path.join(pluginPath, 'images', 'input.svg'))
             self.picture = QPicture()
@@ -175,6 +173,31 @@ class ModelerGraphicItem(QGraphicsItem):
     def mouseDoubleClickEvent(self, event):
         self.editElement()
 
+    def hoverEnterEvent(self, event):
+        self.updateToolTip(event)
+
+    def hoverMoveEvent(self, event):
+        self.updateToolTip(event)
+
+    def hoverLeaveEvent(self, event):
+        self.setToolTip('')
+        if self.hover_over_item:
+            self.hover_over_item = False
+            self.update()
+            self.repaintArrows()
+
+    def updateToolTip(self, event):
+        prev_status = self.hover_over_item
+        if self.itemRect().contains(event.pos()):
+            self.setToolTip(self.text)
+            self.hover_over_item = True
+        else:
+            self.setToolTip('')
+            self.hover_over_item = False
+        if self.hover_over_item != prev_status:
+            self.update()
+            self.repaintArrows()
+
     def contextMenuEvent(self, event):
         if isinstance(self.element, QgsProcessingModelOutput):
             return
@@ -291,12 +314,15 @@ class ModelerGraphicItem(QGraphicsItem):
             w = fm.width(text)
         return text
 
-    def paint(self, painter, option, widget=None):
-        rect = QRectF(-(ModelerGraphicItem.BOX_WIDTH + 2) / 2.0,
+    def itemRect(self):
+        return QRectF(-(ModelerGraphicItem.BOX_WIDTH + 2) / 2.0,
                       -(ModelerGraphicItem.BOX_HEIGHT + 2) / 2.0,
                       ModelerGraphicItem.BOX_WIDTH + 2,
                       ModelerGraphicItem.BOX_HEIGHT + 2)
 
+    def paint(self, painter, option, widget=None):
+        rect = self.itemRect()
+
         if isinstance(self.element, QgsProcessingModelParameter):
             color = QColor(238, 242, 131)
             stroke = QColor(234, 226, 118)
@@ -312,6 +338,8 @@ class ModelerGraphicItem(QGraphicsItem):
         if self.isSelected():
             stroke = selected
             color = color.darker(110)
+        if self.hover_over_item:
+            color = color.darker(105)
         painter.setPen(QPen(stroke, 0))  # 0 width "cosmetic" pen
         painter.setBrush(QBrush(color, Qt.SolidPattern))
         painter.drawRect(rect)
@@ -328,7 +356,7 @@ class ModelerGraphicItem(QGraphicsItem):
         h = fm.ascent()
         pt = QPointF(-ModelerGraphicItem.BOX_WIDTH / 2 + 25, ModelerGraphicItem.BOX_HEIGHT / 2.0 - h + 1)
         painter.drawText(pt, text)
-        painter.setPen(QPen(Qt.black))
+        painter.setPen(QPen(QApplication.palette().color(QPalette.WindowText)))
         if isinstance(self.element, QgsProcessingModelChildAlgorithm):
             h = -(fm.height() * 1.2)
             h = h - ModelerGraphicItem.BOX_HEIGHT / 2.0 + 5
@@ -397,6 +425,10 @@ class ModelerGraphicItem(QGraphicsItem):
         else:
             return QPointF(0, 0)
 
+    def repaintArrows(self):
+        for arrow in self.arrows:
+            arrow.update()
+
     def itemChange(self, change, value):
         if change == QGraphicsItem.ItemPositionHasChanged:
             for arrow in self.arrows:
@@ -410,8 +442,10 @@ class ModelerGraphicItem(QGraphicsItem):
                 self.model.parameterComponent(self.element.parameterName()).setPosition(self.pos())
             elif isinstance(self.element, QgsProcessingModelOutput):
                 self.model.childAlgorithm(self.element.childId()).modelOutput(self.element.name()).setPosition(self.pos())
+        elif change == QGraphicsItem.ItemSelectedChange:
+            self.repaintArrows()
 
-        return value
+        return super().itemChange(change, value)
 
     def polygon(self):
         font = QFont('Verdana', 8)
