@@ -23,19 +23,28 @@
  ***************************************************************************/
 """
 
-from PyQt4.QtCore import Qt, QObject, QSettings, QDir, QUrl
-from PyQt4.QtGui import QMessageBox, QLabel, QFrame, QApplication
-from PyQt4.QtNetwork import QNetworkRequest
+import os
+import json
+import zipfile
+
+from qgis.PyQt.QtCore import Qt, QObject, QDir, QUrl, QFileInfo, QFile
+from qgis.PyQt.QtWidgets import QApplication, QDialog, QDialogButtonBox, QFrame, QMessageBox, QLabel, QVBoxLayout
+from qgis.PyQt.QtNetwork import QNetworkRequest
 
 import qgis
-from qgis.core import QgsApplication, QgsNetworkAccessManager
-from qgis.gui import QgsMessageBar
-from qgis.utils import iface, startPlugin, unloadPlugin, loadPlugin, reloadPlugin, updateAvailablePlugins
-from installer_data import repositories, plugins, officialRepo, settingsGroup, reposGroup, removeDir
-from qgsplugininstallerinstallingdialog import QgsPluginInstallerInstallingDialog
-from qgsplugininstallerpluginerrordialog import QgsPluginInstallerPluginErrorDialog
-from qgsplugininstallerfetchingdialog import QgsPluginInstallerFetchingDialog
-from qgsplugininstallerrepositorydialog import QgsPluginInstallerRepositoryDialog
+from qgis.core import Qgis, QgsApplication, QgsNetworkAccessManager, QgsSettings, QgsNetworkRequestParameters
+from qgis.gui import QgsMessageBar, QgsPasswordLineEdit
+from qgis.utils import (iface, startPlugin, unloadPlugin, loadPlugin,
+                        reloadPlugin, updateAvailablePlugins, plugins_metadata_parser)
+from .installer_data import (repositories, plugins, officialRepo,
+                             settingsGroup, reposGroup, removeDir)
+from .qgsplugininstallerinstallingdialog import QgsPluginInstallerInstallingDialog
+from .qgsplugininstallerpluginerrordialog import QgsPluginInstallerPluginErrorDialog
+from .qgsplugininstallerfetchingdialog import QgsPluginInstallerFetchingDialog
+from .qgsplugininstallerrepositorydialog import QgsPluginInstallerRepositoryDialog
+from .unzip import unzip
+from .plugindependencies import find_dependencies
+from .qgsplugindependenciesdialog import QgsPluginDependenciesDialog
 
 
 # public instances:
@@ -58,14 +67,14 @@ class QgsPluginInstaller(QObject):
     def __init__(self):
         """ Initialize data objects, starts fetching if appropriate, and warn about/removes obsolete plugins """
 
-        QObject.__init__(self) # initialize QObject in order to to use self.tr()
+        QObject.__init__(self)  # initialize QObject in order to to use self.tr()
         repositories.load()
         plugins.getAllInstalled()
 
         if repositories.checkingOnStart() and repositories.timeForChecking() and repositories.allEnabled():
             # start fetching repositories
-            self.statusLabel = QLabel(self.tr("Looking for new plugins...") + " ", iface.mainWindow().statusBar())
-            iface.mainWindow().statusBar().insertPermanentWidget(0, self.statusLabel)
+            self.statusLabel = QLabel(iface.mainWindow().statusBar())
+            iface.mainWindow().statusBar().addPermanentWidget(self.statusLabel)
             self.statusLabel.linkActivated.connect(self.showPluginManagerWhenReady)
             repositories.checkingDone.connect(self.checkingDone)
             for key in repositories.allEnabled():
@@ -75,7 +84,7 @@ class QgsPluginInstaller(QObject):
             for key in repositories.allEnabled():
                 repositories.setRepositoryData(key, "state", 3)
 
-        # look for obsolete plugins (the user-installed one is newer than core one)
+        # look for obsolete plugins updates (the user-installed one is older than the core one)
         for key in plugins.obsoletePlugins:
             plugin = plugins.localCache[key]
             msg = QMessageBox()
@@ -86,10 +95,10 @@ class QgsPluginInstaller(QObject):
             msg.setText("%s <b>%s</b><br/><br/>%s" % (self.tr("Obsolete plugin:"), plugin["name"], self.tr("QGIS has detected an obsolete plugin that masks its more recent version shipped with this copy of QGIS. This is likely due to files associated with a previous installation of QGIS. Do you want to remove the old plugin right now and unmask the more recent version?")))
             msg.exec_()
             if not msg.result():
-                # uninstall, update utils and reload if enabled
+                # uninstall the update, update utils and reload if enabled
                 self.uninstallPlugin(key, quiet=True)
                 updateAvailablePlugins()
-                settings = QSettings()
+                settings = QgsSettings()
                 if settings.value("/PythonPlugins/" + key, False, type=bool):
                     settings.setValue("/PythonPlugins/watchDog/" + key, True)
                     loadPlugin(key)
@@ -99,7 +108,7 @@ class QgsPluginInstaller(QObject):
     # ----------------------------------------- #
     def fetchAvailablePlugins(self, reloadMode):
         """ Fetch plugins from all enabled repositories."""
-        """  reloadMode = true:  Fully refresh data from QSettings to mRepositories  """
+        """  reloadMode = true:  Fully refresh data from QgsSettings to mRepositories  """
         """  reloadMode = false: Fetch unready repositories only """
         QApplication.setOverrideCursor(Qt.WaitCursor)
 
@@ -109,7 +118,7 @@ class QgsPluginInstaller(QObject):
             plugins.getAllInstalled()
 
         for key in repositories.allEnabled():
-            if reloadMode or repositories.all()[key]["state"] == 3: # if state = 3 (error or not fetched yet), try to fetch once again
+            if reloadMode or repositories.all()[key]["state"] == 3:  # if state = 3 (error or not fetched yet), try to fetch once again
                 repositories.requestFetching(key)
 
         if repositories.fetchingInProgress():
@@ -121,7 +130,7 @@ class QgsPluginInstaller(QObject):
 
         QApplication.restoreOverrideCursor()
 
-        # display error messages for every unavailable reposioty, unless Shift pressed nor all repositories are unavailable
+        # display error messages for every unavailable repository, unless Shift pressed nor all repositories are unavailable
         keepQuiet = QgsApplication.keyboardModifiers() == Qt.KeyboardModifiers(Qt.ShiftModifier)
         if repositories.allUnavailable() and repositories.allUnavailable() != repositories.allEnabled():
             for key in repositories.allUnavailable():
@@ -143,19 +152,23 @@ class QgsPluginInstaller(QObject):
         # look for news in the repositories
         plugins.markNews()
         status = ""
+        icon = ""
         # first check for news
         for key in plugins.all():
             if plugins.all()[key]["status"] == "new":
                 status = self.tr("There is a new plugin available")
-                tabIndex = 4 # PLUGMAN_TAB_NEW
+                icon = "pluginNew.svg"
+                tabIndex = 4  # PLUGMAN_TAB_NEW
         # then check for updates (and eventually overwrite status)
         for key in plugins.all():
             if plugins.all()[key]["status"] == "upgradeable":
                 status = self.tr("There is a plugin update available")
-                tabIndex = 3 # PLUGMAN_TAB_UPGRADEABLE
+                icon = "pluginUpgrade.svg"
+                tabIndex = 3  # PLUGMAN_TAB_UPGRADEABLE
         # finally set the notify label
         if status:
-            self.statusLabel.setText(u' <a href="%d">%s</a>  ' % (tabIndex, status))
+            self.statusLabel.setText(u'<a href="%d"><img src="qrc:/images/themes/default/%s"></a>' % (tabIndex, icon))
+            self.statusLabel.setToolTip(status)
         else:
             iface.mainWindow().statusBar().removeWidget(self.statusLabel)
             self.statusLabel = None
@@ -211,6 +224,7 @@ class QgsPluginInstaller(QObject):
                 "error_details": plugin["error_details"],
                 "experimental": plugin["experimental"] and "true" or "false",
                 "deprecated": plugin["deprecated"] and "true" or "false",
+                "trusted": plugin["trusted"] and "true" or "false",
                 "version_available": plugin["version_available"],
                 "zip_repository": plugin["zip_repository"],
                 "download_url": plugin["download_url"],
@@ -218,6 +232,7 @@ class QgsPluginInstaller(QObject):
                 "downloads": plugin["downloads"],
                 "average_vote": plugin["average_vote"],
                 "rating_votes": plugin["rating_votes"],
+                "plugin_dependencies": plugin.get("plugin_dependencies", None),
                 "pythonic": "true"
             })
         iface.pluginManagerInterface().reloadModel()
@@ -244,7 +259,7 @@ class QgsPluginInstaller(QObject):
         # finally, show the plugin manager window
         tabIndex = -1
         if len(params) == 1:
-            indx = unicode(params[0])
+            indx = str(params[0])
             if indx.isdigit() and int(indx) > -1 and int(indx) < 7:
                 tabIndex = int(indx)
         iface.pluginManagerInterface().showPluginManager(tabIndex)
@@ -257,7 +272,7 @@ class QgsPluginInstaller(QObject):
 
     # ----------------------------------------- #
     def exportSettingsGroup(self):
-        """ Return QSettings settingsGroup value """
+        """ Return QgsSettings settingsGroup value """
         return settingsGroup
 
     # ----------------------------------------- #
@@ -275,7 +290,7 @@ class QgsPluginInstaller(QObject):
         previousStatus = plugin["status"]
         if not plugin:
             return
-        if plugin["status"] == "newer" and not plugin["error"]: # ask for confirmation if user downgrades an usable plugin
+        if plugin["status"] == "newer" and not plugin["error"]:  # ask for confirmation if user downgrades an usable plugin
             if QMessageBox.warning(iface.mainWindow(), self.tr("QGIS Python Plugin Installer"), self.tr("Are you sure you want to downgrade the plugin to the latest available version? The installed one is newer!"), QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
                 return
 
@@ -297,24 +312,25 @@ class QgsPluginInstaller(QObject):
             QApplication.setOverrideCursor(Qt.WaitCursor)
             # update the list of plugins in plugin handling routines
             updateAvailablePlugins()
+            self.processDependencies(plugin["id"])
             # try to load the plugin
             loadPlugin(plugin["id"])
-            plugins.getAllInstalled(testLoad=True)
+            plugins.getAllInstalled()
             plugins.rebuild()
             plugin = plugins.all()[key]
             if not plugin["error"]:
                 if previousStatus in ["not installed", "new"]:
                     infoString = (self.tr("Plugin installed successfully"), "")
                     if startPlugin(plugin["id"]):
-                        settings = QSettings()
+                        settings = QgsSettings()
                         settings.setValue("/PythonPlugins/" + plugin["id"], True)
                 else:
-                    settings = QSettings()
-                    if settings.value("/PythonPlugins/" + key, False, type=bool): # plugin will be reloaded on the fly only if currently loaded
-                        reloadPlugin(key) # unloadPlugin + loadPlugin + startPlugin
+                    settings = QgsSettings()
+                    if settings.value("/PythonPlugins/" + key, False, type=bool):  # plugin will be reloaded on the fly only if currently loaded
+                        reloadPlugin(key)  # unloadPlugin + loadPlugin + startPlugin
                         infoString = (self.tr("Plugin reinstalled successfully"), "")
                     else:
-                        unloadPlugin(key) # Just for a case. Will exit quietly if really not loaded
+                        unloadPlugin(key)  # Just for a case. Will exit quietly if really not loaded
                         loadPlugin(key)
                         infoString = (self.tr("Plugin reinstalled successfully"), self.tr("Python plugin reinstalled.\nYou need to restart QGIS in order to reload it."))
                 if quiet:
@@ -341,14 +357,14 @@ class QgsPluginInstaller(QObject):
                         error = True
                         infoString = (self.tr("Plugin uninstall failed"), result)
                         try:
-                            exec ("sys.path_importer_cache.clear()")
-                            exec ("import %s" % plugin["id"])
-                            exec ("reload (%s)" % plugin["id"])
+                            exec("sys.path_importer_cache.clear()")
+                            exec("import %s" % plugin["id"])
+                            exec("reload (%s)" % plugin["id"])
                         except:
                             pass
                     else:
                         try:
-                            exec ("del sys.modules[%s]" % plugin["id"])
+                            exec("del sys.modules[%s]" % plugin["id"])
                         except:
                             pass
                     plugins.getAllInstalled()
@@ -357,8 +373,10 @@ class QgsPluginInstaller(QObject):
             self.exportPluginsToManager()
 
         if infoString[0]:
-            level = error and QgsMessageBar.CRITICAL or QgsMessageBar.INFO
-            msg = "<b>%s:</b>%s" % (infoString[0], infoString[1])
+            level = error and Qgis.Critical or Qgis.Info
+            msg = "<b>%s</b>" % infoString[0]
+            if infoString[1]:
+                msg += "<b>:</b> %s" % infoString[1]
             iface.pluginManagerInterface().pushMessage(msg, level)
 
     # ----------------------------------------- #
@@ -387,7 +405,7 @@ class QgsPluginInstaller(QObject):
         if result:
             QApplication.restoreOverrideCursor()
             msg = "<b>%s:</b>%s" % (self.tr("Plugin uninstall failed"), result)
-            iface.pluginManagerInterface().pushMessage(msg, QgsMessageBar.CRITICAL)
+            iface.pluginManagerInterface().pushMessage(msg, Qgis.Critical)
         else:
             # safe remove
             try:
@@ -395,19 +413,24 @@ class QgsPluginInstaller(QObject):
             except:
                 pass
             try:
-                exec ("plugins[%s].unload()" % plugin["id"])
-                exec ("del plugins[%s]" % plugin["id"])
+                exec("plugins[%s].unload()" % plugin["id"])
+                exec("del plugins[%s]" % plugin["id"])
             except:
                 pass
             try:
-                exec ("del sys.modules[%s]" % plugin["id"])
+                exec("del sys.modules[%s]" % plugin["id"])
             except:
                 pass
+            try:
+                exec("del plugins_metadata_parser[%s]" % plugin["id"])
+            except:
+                pass
+
             plugins.getAllInstalled()
             plugins.rebuild()
             self.exportPluginsToManager()
             QApplication.restoreOverrideCursor()
-            iface.pluginManagerInterface().pushMessage(self.tr("Plugin uninstalled successfully"), QgsMessageBar.INFO)
+            iface.pluginManagerInterface().pushMessage(self.tr("Plugin uninstalled successfully"), Qgis.Info)
 
     # ----------------------------------------- #
     def addRepository(self):
@@ -417,11 +440,11 @@ class QgsPluginInstaller(QObject):
         dlg.checkBoxEnabled.setCheckState(Qt.Checked)
         if not dlg.exec_():
             return
-        for i in repositories.all().values():
+        for i in list(repositories.all().values()):
             if dlg.editURL.text().strip() == i["url"]:
-                iface.pluginManagerInterface().pushMessage(self.tr("Unable to add another repository with the same URL!"), QgsMessageBar.WARNING)
+                iface.pluginManagerInterface().pushMessage(self.tr("Unable to add another repository with the same URL!"), Qgis.Warning)
                 return
-        settings = QSettings()
+        settings = QgsSettings()
         settings.beginGroup(reposGroup)
         reposName = dlg.editName.text()
         reposURL = dlg.editURL.text().strip()
@@ -440,7 +463,6 @@ class QgsPluginInstaller(QObject):
         """ edit repository connection """
         if not reposName:
             return
-        reposName = reposName.decode('utf-8')
         checkState = {False: Qt.Unchecked, True: Qt.Checked}
         dlg = QgsPluginInstallerRepositoryDialog(iface.mainWindow())
         dlg.editName.setText(reposName)
@@ -456,13 +478,13 @@ class QgsPluginInstaller(QObject):
             dlg.labelInfo.setText(self.tr("This repository is blocked due to incompatibility with your QGIS version"))
             dlg.labelInfo.setFrameShape(QFrame.Box)
         if not dlg.exec_():
-            return # nothing to do if cancelled
-        for i in repositories.all().values():
+            return  # nothing to do if canceled
+        for i in list(repositories.all().values()):
             if dlg.editURL.text().strip() == i["url"] and dlg.editURL.text().strip() != repositories.all()[reposName]["url"]:
-                iface.pluginManagerInterface().pushMessage(self.tr("Unable to add another repository with the same URL!"), QgsMessageBar.WARNING)
+                iface.pluginManagerInterface().pushMessage(self.tr("Unable to add another repository with the same URL!"), Qgis.Warning)
                 return
-        # delete old repo from QSettings and create new one
-        settings = QSettings()
+        # delete old repo from QgsSettings and create new one
+        settings = QgsSettings()
         settings.beginGroup(reposGroup)
         settings.remove(reposName)
         newName = dlg.editName.text()
@@ -476,7 +498,7 @@ class QgsPluginInstaller(QObject):
         if dlg.editURL.text().strip() == repositories.all()[reposName]["url"] and dlg.checkBoxEnabled.checkState() == checkState[repositories.all()[reposName]["enabled"]]:
             repositories.rename(reposName, newName)
             self.exportRepositoriesToManager()
-            return # nothing else to do if only repository name was changed
+            return  # nothing else to do if only repository name was changed
         plugins.removeRepository(reposName)
         self.reloadAndExportData()
 
@@ -485,11 +507,10 @@ class QgsPluginInstaller(QObject):
         """ delete repository connection """
         if not reposName:
             return
-        reposName = reposName.decode('utf-8')
-        settings = QSettings()
+        settings = QgsSettings()
         settings.beginGroup(reposGroup)
-        if settings.value(reposName + "/url", "", type=unicode) == officialRepo[1]:
-            iface.pluginManagerInterface().pushMessage(self.tr("You can't remove the official QGIS Plugin Repository. You can disable it if needed."), QgsMessageBar.WARNING)
+        if settings.value(reposName + "/url", "", type=str) == officialRepo[1]:
+            iface.pluginManagerInterface().pushMessage(self.tr("You can't remove the official QGIS Plugin Repository. You can disable it if needed."), Qgis.Warning)
             return
         warning = self.tr("Are you sure you want to remove the following repository?") + "\n" + reposName
         if QMessageBox.warning(iface.mainWindow(), self.tr("QGIS Python Plugin Installer"), warning, QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
@@ -503,8 +524,6 @@ class QgsPluginInstaller(QObject):
     # ----------------------------------------- #
     def setRepositoryInspectionFilter(self, reposName=None):
         """ temporarily block another repositories to fetch only one for inspection """
-        if reposName is not None:
-            reposName = reposName.decode("utf-8")
         repositories.setInspectionFilter(reposName)
         self.reloadAndExportData()
 
@@ -515,8 +534,129 @@ class QgsPluginInstaller(QObject):
         if not plugin_id or not vote:
             return False
         url = "http://plugins.qgis.org/plugins/RPC2/"
-        params = "{\"id\":\"djangorpc\",\"method\":\"plugin.vote\",\"params\":[%s,%s]}" % (str(plugin_id), str(vote))
+        params = {"id": "djangorpc", "method": "plugin.vote", "params": [str(plugin_id), str(vote)]}
         req = QNetworkRequest(QUrl(url))
-        req.setRawHeader("Content-Type", "application/json")
-        QgsNetworkAccessManager.instance().post(req, params)
+        req.setAttribute(QNetworkRequest.Attribute(QgsNetworkRequestParameters.AttributeInitiatorClass), "QgsPluginInstaller")
+        req.setAttribute(QNetworkRequest.Attribute(QgsNetworkRequestParameters.AttributeInitiatorRequestId), "sendVote")
+        req.setRawHeader(b"Content-Type", b"application/json")
+        QgsNetworkAccessManager.instance().post(req, bytes(json.dumps(params), "utf-8"))
         return True
+
+    def installFromZipFile(self, filePath):
+        if not os.path.isfile(filePath):
+            return
+
+        settings = QgsSettings()
+        settings.setValue(settingsGroup + '/lastZipDirectory',
+                          QFileInfo(filePath).absoluteDir().absolutePath())
+
+        with zipfile.ZipFile(filePath, 'r') as zf:
+            pluginName = os.path.split(zf.namelist()[0])[0]
+
+        pluginFileName = os.path.splitext(os.path.basename(filePath))[0]
+
+        pluginsDirectory = qgis.utils.home_plugin_path
+        if not QDir(pluginsDirectory).exists():
+            QDir().mkpath(pluginsDirectory)
+
+        pluginDirectory = QDir.cleanPath(os.path.join(pluginsDirectory, pluginName))
+
+        # If the target directory already exists as a link,
+        # remove the link without resolving
+        QFile(pluginDirectory).remove()
+
+        password = None
+        infoString = None
+        success = False
+        keepTrying = True
+
+        while keepTrying:
+            try:
+                # Test extraction. If fails, then exception will be raised and no removing occurs
+                unzip(filePath, pluginsDirectory, password)
+                # Removing old plugin files if exist
+                removeDir(pluginDirectory)
+                # Extract new files
+                unzip(filePath, pluginsDirectory, password)
+                keepTrying = False
+                success = True
+            except Exception as e:
+                success = False
+                if 'password' in str(e):
+                    infoString = self.tr('Aborted by user')
+                    if 'Bad password' in str(e):
+                        msg = self.tr('Wrong password. Please enter a correct password to the zip file.')
+                    else:
+                        msg = self.tr('The zip file is encrypted. Please enter password.')
+                    # Display a password dialog with QgsPasswordLineEdit
+                    dlg = QDialog()
+                    dlg.setWindowTitle(self.tr('Enter password'))
+                    buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel, Qt.Horizontal)
+                    buttonBox.rejected.connect(dlg.reject)
+                    buttonBox.accepted.connect(dlg.accept)
+                    lePass = QgsPasswordLineEdit()
+                    layout = QVBoxLayout()
+                    layout.addWidget(QLabel(msg))
+                    layout.addWidget(lePass)
+                    layout.addWidget(buttonBox)
+                    dlg.setLayout(layout)
+                    keepTrying = dlg.exec_()
+                    password = lePass.text()
+                else:
+                    infoString = self.tr("Failed to unzip the plugin package\n{}.\nProbably it is broken".format(filePath))
+                    keepTrying = False
+
+        if success:
+            updateAvailablePlugins()
+            self.processDependencies(pluginName)
+            loadPlugin(pluginName)
+            plugins.getAllInstalled()
+            plugins.rebuild()
+
+            if settings.contains('/PythonPlugins/' + pluginName):
+                if settings.value('/PythonPlugins/' + pluginName, False, bool):
+                    startPlugin(pluginName)
+                    reloadPlugin(pluginName)
+                else:
+                    unloadPlugin(pluginName)
+                    loadPlugin(pluginName)
+            else:
+                if startPlugin(pluginName):
+                    settings.setValue('/PythonPlugins/' + pluginName, True)
+
+            self.exportPluginsToManager()
+            msg = "<b>%s</b>" % self.tr("Plugin installed successfully")
+        else:
+            msg = "<b>%s:</b> %s" % (self.tr("Plugin installation failed"), infoString)
+
+        level = Qgis.Info if success else Qgis.Critical
+        iface.pluginManagerInterface().pushMessage(msg, level)
+
+    def processDependencies(self, plugin_id):
+        """Processes plugin dependencies
+
+        :param plugin_id: plugin id
+        :type plugin_id: str
+        """
+
+        to_install, to_upgrade, not_found = find_dependencies(plugin_id)
+        if to_install or to_upgrade or not_found:
+            dlg = QgsPluginDependenciesDialog(plugin_id, to_install, to_upgrade, not_found)
+            if dlg.exec_() == QgsPluginDependenciesDialog.Accepted:
+                actions = dlg.actions()
+                for dependency_plugin_id, action in actions.items():
+                    try:
+                        self.installPlugin(dependency_plugin_id)
+                        if action == 'install':
+                            iface.pluginManagerInterface().pushMessage(self.tr("Plugin dependency <b>%s</b> successfully installed") %
+                                                                       dependency_plugin_id, Qgis.Info)
+                        else:
+                            iface.pluginManagerInterface().pushMessage(self.tr("Plugin dependency <b>%s</b> successfully upgraded") %
+                                                                       dependency_plugin_id, Qgis.Info)
+                    except Exception as ex:
+                        if action == 'install':
+                            iface.pluginManagerInterface().pushMessage(self.tr("Error installing plugin dependency <b>%s</b>: %s") %
+                                                                       (dependency_plugin_id, ex), Qgis.Warning)
+                        else:
+                            iface.pluginManagerInterface().pushMessage(self.tr("Error upgrading plugin dependency <b>%s</b>: %s") %
+                                                                       (dependency_plugin_id, ex), Qgis.Warning)
